# Implementation in R 

This is an R function to encapsulate these equations, `transformstats` automates transformations of SD, MSE, LSD, 95\%CI, HSD, and MSD to conservative estimates of SE, and is used by PEcAn \cite{LeBauer_2013}. The `se.est` function estimates SE through simulation, but is not currently implemented within PEcAn, but has not been implemented.

## Arithmetic Transformations

The `transformstats` function:


    transformstats <- function(data) {
      if(!"SE" %in% levels(data$statname)){
        data$statname <- factor(data$statname, levels = c(levels(data$statname), "SE"))
      }
      ## Transformation of stats to SE
      ## transform SD to SE
      if (max(c("SD","sd") %in% data$statname)) {
        sdi <- which(data$statname %in% c("SD","sd"))
        data$stat[sdi] <- data$stat[sdi] / sqrt(data$n[sdi])
        data$statname[sdi] <- "SE"
      }
      ## transform MSE to SE
      if ("MSE" %in% data$statname) {
        msei <- which(data$statname == "MSE")
        data$stat[msei] <- sqrt (data$stat[msei]/data$n[msei])
        data$statname[msei] <- "SE"
      }
      ## 95%CI measured from mean to upper or lower CI
      ## SE = CI/t
      if ("95%CI" %in% data$statname) {
        cii <- which(data$statname == '95%CI')
        data$stat[cii] <- data$stat[cii]/qt(0.975,data$n[cii])
        data$statname[cii] <- "SE"
      }
      ## Fisher's Least Significant Difference (LSD)
      ## conservatively assume no within block replication
      if ("LSD" %in% data$statname) {
        lsdi <- which(data$statname == "LSD")
        data$stat[lsdi] <- data$stat[lsdi] / (qt(0.975,data$n[lsdi]) * sqrt( (2 * data$n[lsdi])))
        data$statname[lsdi] <- "SE"
      }
      ## Tukey's Honestly Significant Difference (HSD),
      ## conservatively assuming 3 groups being tested so df =2
      if ("HSD" %in% data$statname) {
        hsdi <- which(data$statname == "HSD")
        n = data$n[hsdi]
        n[is.na(n)] = 2 ## minimum n that can be used if NA
        data$stat[hsdi] <- data$stat[hsdi] / (qtukey(0.975, n, df = 2))
        data$statname[hsdi] <- "SE"
        data$n[hsdi] <- n
      }              
      ## MSD Minimum Squared Difference
      ## MSD = t_{\alpha/2, 2n-2}*SD*sqrt(2/n)
      ## SE  = MSD*n/(t*sqrt(2))
      if ("MSD" %in% data$statname) {
        msdi <- which(data$statname == "MSD")
        data$stat[msdi] <- data$stat[msdi] * data$n[msdi] / ( qt(0.975,2*data$n[msdi]-2)*sqrt(2))
        data$statname[msdi] <- "SE"
      }
      if (FALSE %in% c('SE','none') %in% data$statname) {
        print(paste(trait, ': ERROR!!! data contains untransformed statistics'))
      }
      return(data)
    }

Example Use:

    statdf <- data.frame(Y=rep(1,5),
        stat=rep(1,5),
        n=rep(4,5),
        statname=c('SD', 'MSE', 'LSD', 'HSD', 'MSD'))
    transformstats(statdf)
    
    
## Transformation via simulation

As an alternative to solving for \(SE\) given \(n\), \(\bar{X_i}\) and summary statistics from post-hoc multiple comparisons such as Fisher's \(LSD\) and Tukey's \(HSD\), I propose a general approach that can be applied to other post-hoc tests as well as other statistics. Ultimately, such an approach would be less prone to (my own) arithmetic error and easier to implement than trying to analytically solve each new statistic. In practice, this has not been implemented because the analytical solutions above cover the majority of cases. However, this could be a good way to test some of the analytical solutions. 

The goal here is to generalize the estimation of \(SE\) from other statistics by simulating the assumed data generating models. 

Is there a general simulation / optimization approach to this problem?

 - \(\bar{X_1}\), \(\bar{X_2}\), some statistic, and n are given
 - I am looking for \(SE\pm 10\%\)
 - As in the earlier post, over estimates of SE are o.k.

The following example implementations would be a solution for SE when the LSD is given:

    se.est <- function(x1bar, x2bar, lsd_obs, n, se) {
           y <- c(rnorm(n, x1bar, se), rnorm(n, x2bar, se)).
           x <- c(rep(1,n), rep(2,n))
           mse <- sum(lm(y~factor(x))$residuals^2)/(2*n-2)
           lsd_est <- qt(0.975,n)*sqrt(2*mse/2)
           ans <- (lsd_obs - lsd_est)^2
           }

    SE <- optimize(par = c(x1bar, x2bar, lsd_obs,n), fn = se.est)
  
alternatively

    for (i in 1:10000){
         se[i] <- rgamma(1, 1, 0.01)
         y[i] <- c(rnorm(n, x1bar, se), rnorm(n, x2bar, se)).
         x[i] <- c(rep(1,n), rep(2,n))
         mse[i] <- sum(lm(y~factor(x))$residuals^2)/(2*n-2)
         lsd_est[i] <- qt(0.975,n)*sqrt(2*mse/2)
         }

     se <- se[which.min((lsd_est - lsd_obs)^2)]

